using System.Drawing;
using System.IO;
using Microsoft.VisualStudio.Modeling;
using DslModeling = global::Microsoft.VisualStudio.Modeling;
using DslDesign = global::Microsoft.VisualStudio.Modeling.Design;
using DslDiagrams = global::Microsoft.VisualStudio.Modeling.Diagrams;
using System.Reflection;
using Microsoft.VisualStudio.Modeling.Diagrams;

namespace ABC.Tools.Dsl
{
    public partial class ClassShape : ClassShapeBase
    {
        /// <summary>
        /// Gets an array of CompartmentMappings for all compartments displayed on this shape
        /// (including compartment maps defined on base shapes). 
        /// </summary>
        /// <param name="melType">The type of the DomainClass that this shape is mapped to</param>
        /// <returns></returns>
        protected override DslDiagrams::CompartmentMapping[] GetCompartmentMappings(global::System.Type melType)
        {
            if (melType == null) throw new global::System.ArgumentNullException("melType");

            CompartmentMapping[] mappings = base.GetCompartmentMappings(melType);

            foreach (CompartmentMapping mapping in mappings)
            {
                ElementListCompartmentMapping elemMap = mapping as ElementListCompartmentMapping;

                if (elemMap != null)
                {
                    if (elemMap.CompartmentId == "AttributesCompartment")
                    { elemMap.ImageGetter = AttributeDisplayImageGetter; }
                    if (elemMap.CompartmentId == "OperationsCompartment")
                    { elemMap.ImageGetter = OperationDisplayImageGetter; }
                }
            }

            return mappings;
        }

        //This will deliver custom images to operations
        public Image OperationDisplayImageGetter(ModelElement element)
        {
            Assembly a = Assembly.GetExecutingAssembly();
            Stream imgStream = a.GetManifestResourceStream("ABC.Tools.CustomResources.Operation.bmp");
            return Image.FromStream(imgStream);
        }

        //This will deliver custom images to attributes
        public Image AttributeDisplayImageGetter(ModelElement element)
        {
            Assembly a = Assembly.GetExecutingAssembly();
            Stream imgStream;
            if(element is StatefulAttribute)
                imgStream = a.GetManifestResourceStream("ABC.Tools.CustomResources.StatefulAttribute.bmp");
            else
                imgStream = a.GetManifestResourceStream("ABC.Tools.CustomResources.Attribute.bmp");
            return Image.FromStream(imgStream);
        }
    }

    public partial class InterfaceShape : InterfaceShapeBase
    {
        /// <summary>
        /// Gets an array of CompartmentMappings for all compartments displayed on this shape
        /// (including compartment maps defined on base shapes). 
        /// </summary>
        /// <param name="melType">The type of the DomainClass that this shape is mapped to</param>
        /// <returns></returns>
        protected override DslDiagrams::CompartmentMapping[] GetCompartmentMappings(global::System.Type melType)
        {
            if (melType == null) throw new global::System.ArgumentNullException("melType");

            CompartmentMapping[] mappings = base.GetCompartmentMappings(melType);

            foreach (CompartmentMapping mapping in mappings)
            {
                ElementListCompartmentMapping elemMap = mapping as ElementListCompartmentMapping;

                if (elemMap != null)
                {
                    if (elemMap.CompartmentId == "OperationsCompartment")
                    { elemMap.ImageGetter = OperationDisplayImageGetter; }
                }
            }

            return mappings;
        }

        //This will deliver custom images to operations
        public Image OperationDisplayImageGetter(ModelElement element)
        {
            Assembly a = Assembly.GetExecutingAssembly();
            Stream imgStream = a.GetManifestResourceStream("ABC.Tools.CustomResources.Operation.bmp");
            return Image.FromStream(imgStream);
        }
    }
}
